--
-- rotomaster
--
-- @author  Manuel L. (1.LS-Modschmiede)
-- @date  01/08/08

rotomaster = {};

function rotomaster:new(configFile, positionX, offsetY, positionZ, rotationY, customMt)

    if rotomaster_mt == nil then
        rotomaster_mt = Class(rotomaster, Trailer);
    end;

    local mt = customMt;
    if mt == nil then
        mt = rotomaster_mt;
    end;
    local instance = Trailer:new(configFile, positionX, offsetY, positionZ, rotationY, mt);

    local xmlFile = loadXMLFile("TempConfig", configFile);

    instance.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
	
	instance.drum1= Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.drum#index1"));
	instance.drum2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.drum#index2"));
	instance.drum3 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.drum#index3"));
    instance.drumRotationScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.drum#rotationScale"), 1);	
	
	local pickupNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.pickup#index"));
    if pickupNode ~= nil then
        instance.pickup = {};
        instance.pickup.node = pickupNode;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#minRot"));
        instance.pickup.minRot = {};
        instance.pickup.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.pickup.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.pickup.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#maxRot"));
        instance.pickup.maxRot = {};
        instance.pickup.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.pickup.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.pickup.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.pickup.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#rotTime"), 2)*1000;
        instance.pickup.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#touchRotLimit"), 10));
    end;
	
	instance.trailingTruckBackAngle = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailingTruck#backAngle"), 25);
	instance.trailingTruckActive = true;

	instance.isActive = false;
	instance.hasGroundContact = false;
	
    delete(xmlFile);

    instance.wasToFast = false;

    return instance;
end;

function rotomaster:delete()
    Trailer.delete(self);
end;

function rotomaster:mouseEvent(posX, posY, isDown, isUp, button)
    Trailer.mouseEvent(self, posX, posY, isDown, isUp, button);
end;

function rotomaster:keyEvent(unicode, sym, modifier, isDown)
    Trailer.keyEvent(self, unicode, sym, modifier, isDown);
	if isDown and sym == Input.KEY_n then
		self.isActive = not self.isActive;
	elseif isDown and sym == Input.KEY_m then
		self.hasGroundContact = not self.hasGroundContact;
	end;
	if isDown and sym == Input.KEY_k then
		self.trailingTruckActive = not self.trailingTruckActive;
	end;

end;

function rotomaster:update(dt)
	Trailer.update(self, dt);
	local activeGround = true;
	
	if self.pickup ~= nil then
		local x, y, z = getRotation(self.pickup.node);
		local minRot = self.pickup.minRot;
		local eps = self.pickup.touchRotLimit;
		if math.abs(x-minRot[1]) < eps and math.abs(y-minRot[2]) < eps and math.abs(z-minRot[3]) < eps then
			activeGround = false;
		end;
	end;

	if g_currentMission.currentVehicle ~= nil then
		if g_currentMission.currentVehicle.attachedTrailer == self then
			
			local angle = 0;
			local backAngle = 0;
			
			if self.trailingTruckActive then	
				g_currentMission:addExtraPrintText("Taste K: Lenkachse ausschalten");
			else
				g_currentMission:addExtraPrintText("Taste K: Lenkachse einschalten");
			end;
			
			
			if self.trailingTruckActive then
				local x1,y1,z1 = localDirectionToWorld(self.attacherVehicle.rootNode, 0, 0, 1)
				local x2,y2,z2 = localDirectionToWorld(self.rootNode, 0, 0, 1)

				--2. skalar produkt, wobei y wert ignorieren
				local dot = x1*x2 + z1*z2;

				--3. durch laengen teilen
				dot = dot / (Utils.vector2Length(x1,z1) * Utils.vector2Length(x2,z2))

				--4. arccosine nehmen
				angle = math.acos(dot);
				
				-- y component of cross product
	            local cy = z1 * x2 - x1 * z2;
	            
	            if cy < 0 then
	                angle = -angle;
	            end;
				backAngle  = self.trailingTruckBackAngle * (angle / 50);				
			end;			
			
			for i=1, table.getn(self.wheels) do
		        local x,y,z = getRotation(self.wheels[i].repr);			
				local newX, newY, newZ = getWheelShapePosition(self.wheels[i].node, self.wheels[i].wheelShape);
				setTranslation(self.wheels[i].repr, newX, newY, newZ);
				local axleSpeed = getWheelShapeAxleSpeed(self.wheels[i].node, self.wheels[i].wheelShape)*3.14159/180;
				x = x+axleSpeed*dt/1000.0;
				if i == 2 or i == 4 then
					setWheelShapeProps(self.wheels[i].node, self.wheels[i].wheelShape, 0, 0, backAngle);
					setRotation(self.wheels[i].repr, x, backAngle, z);					
				else
					steeringAngle = 0;
					setRotation(self.wheels[i].repr, x, steeringAngle, z);
				end;
		    end;
			
			
			
			self.wasToFast = false;
		    if self:allowFillType(Trailer.FILLTYPE_GRASS) and self.capacity > self.fillLevel then
				if self.isActive and activeGround then
					local toFast = self.attacherVehicle.lastSpeed*3600 > 29;
			        if not toFast then
			            for i=1, table.getn(self.cuttingAreas) do
			                local x,y,z = getWorldTranslation(self.cuttingAreas[i].start);
			                local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[i].width);
			                local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[i].height);

			                local area = Utils.updateCuttedMeadowArea(x, z, x1, z1, x2, z2);

			                local literPerPixel = 8000/1200 / 6 / (2*2) *12;

			                local deltaLevel = area * literPerPixel * self.fillScale;

			                self:setFillLevel(self.fillLevel+deltaLevel, Trailer.FILLTYPE_GRASS);
			            end;
			        end;
					self.wasToFast = toFast;
				end;
		    end;
			
			if self.drum1 ~= nil and self.isActive then
				rotate(self.drum1, self.drumRotationScale * 35 * -3, 0, 0);   
			end;
			
			if self.drum2 ~= nil and activeGround then
				rotate(self.drum2, self.drumRotationScale * self.attacherVehicle.lastSpeed * self.attacherVehicle.movingDirection * 3 , 0, 0);   
			end;
			
			if self.drum3 ~= nil and activeGround then
				rotate(self.drum3, self.drumRotationScale * self.attacherVehicle.lastSpeed * self.attacherVehicle.movingDirection * 3, 0, 0);   
			end;

			
			if self.pickup ~= nil then
		        local x, y, z = getRotation(self.pickup.node);
		        local rot = {x,y,z};
		        local newRot = Utils.getMovedLimitedValues(rot, self.pickup.maxRot, self.pickup.minRot, 3, self.pickup.rotTime, dt, not self.hasGroundContact);
		        setRotation(self.pickup.node, unpack(newRot));
		    end;

			if self.hasGroundContact then
				g_currentMission:addExtraPrintText("Taste M: Pickup hochziehen");
			else
				g_currentMission:addExtraPrintText("Taste M: Pickup herunterlassen");
			end;		

			if self.isActive then
				g_currentMission:addExtraPrintText("Taste N: Pickup ausschalten");
			else
				g_currentMission:addExtraPrintText("Taste N: Pickup anschalten");
			end;			
		end;
	end;
    

    
end;

function rotomaster:draw()
    Trailer.draw(self);

    if self.wasToFast then
		g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
end;


function rotomaster:onDetach()
    Trailer.onDetach(self);

    self.wasToFast = false;
end;

